@@ -5,8 +5,7 @@ use solana_program::{
     instruction::{AccountMeta, Instruction},
     program_error::ProgramError,
     pubkey::Pubkey,
-    system_program,
-    sysvar,
+    system_program, sysvar,
 };
 use spl_associated_token_account::get_associated_token_address;
 use spl_token;
@@ -64,7 +63,7 @@ pub enum OptionsInstruction {
     ///   13. `[]` SysVar clock account
     ///   14. `[]` System Program account
     ///   
-    MintCoveredCall {},
+    MintCoveredCall { size: u64 },
     /// Exercise an Options token representing a Covered Call
     ///
     ///   0. `[]` Sysvar clock
@@ -79,7 +78,7 @@ pub enum OptionsInstruction {
     ///   9. `[writeable]` Option Mint
     ///   10. `[writeable]` Option Token Account
     ///   11. `[signer]` Option Token Account Authority
-    ExerciseCoveredCall {},
+    ExerciseCoveredCall { size: u64 },
     /// Close a single option contract post expiration.
     /// Transfers the underlying asset back to the Option Writer
     ///
@@ -93,7 +92,7 @@ pub enum OptionsInstruction {
     /// 7. `[writeable]` Underlying Asset Pool
     /// 8. `[]` Sysvar clock
     /// 9. `[]` SPL Token Program
-    ClosePostExpiration {},
+    ClosePostExpiration { size: u64 },
     /// Close a single option contract prior to expiration.
     /// Burns the _option token_ and the _writer token_ and returns the
     /// underlying asset back to the writer (or address specified).
@@ -109,7 +108,7 @@ pub enum OptionsInstruction {
     /// 8. `[]` Writer Token Source Authority
     /// 9. `[writable]` Underlying Asset Destination
     /// 10. `[writable]` Underlying Asset Pool
-    ClosePosition {},
+    ClosePosition { size: u64 },
     /// Allow a user to exchange their Writer Token for Quote Asset.
     /// Burns the Writer Token and transfers the Quote Asset amount
     /// relative to the option market
@@ -123,7 +122,7 @@ pub enum OptionsInstruction {
     /// 6. `[writeable]` Quote Asset Destination
     /// 7. `[writeable]` Quote Asset Pool
     /// 8. `[]` SPL token program
-    ExchangeWriterTokenForQuote {},
+    ExchangeWriterTokenForQuote { size: u64 },
 }
 
 impl OptionsInstruction {
@@ -145,11 +144,26 @@ impl OptionsInstruction {
                     bump_seed,
                 }
             }
-            1 => Self::MintCoveredCall {},
-            2 => Self::ExerciseCoveredCall {},
-            3 => Self::ClosePostExpiration {},
-            4 => Self::ClosePosition {},
-            5 => Self::ExchangeWriterTokenForQuote {},
+            1 => {
+                let (size, _) = Self::unpack_u64(rest)?;
+                Self::MintCoveredCall { size }
+            }
+            2 => {
+                let (size, _) = Self::unpack_u64(rest)?;
+                Self::ExerciseCoveredCall { size }
+            }
+            3 => {
+                let (size, _) = Self::unpack_u64(rest)?;
+                Self::ClosePostExpiration { size }
+            }
+            4 => {
+                let (size, _) = Self::unpack_u64(rest)?;
+                Self::ClosePosition { size }
+            }
+            5 => {
+                let (size, _) = Self::unpack_u64(rest)?;
+                Self::ExchangeWriterTokenForQuote { size }
+            }
             _ => return Err(ProgramError::InvalidInstructionData.into()),
         })
     }
@@ -170,20 +184,25 @@ impl OptionsInstruction {
                 buf.extend_from_slice(&expiration_unix_timestamp.to_le_bytes());
                 buf.extend_from_slice(&bump_seed.to_le_bytes());
             }
-            &Self::MintCoveredCall {} => {
+            &Self::MintCoveredCall { size } => {
                 buf.push(1);
+                buf.extend_from_slice(&size.to_le_bytes());
             }
-            &Self::ExerciseCoveredCall {} => {
+            &Self::ExerciseCoveredCall { size } => {
                 buf.push(2);
+                buf.extend_from_slice(&size.to_le_bytes());
             }
-            &Self::ClosePostExpiration {} => {
+            &Self::ClosePostExpiration { size } => {
                 buf.push(3);
+                buf.extend_from_slice(&size.to_le_bytes());
             }
-            &Self::ClosePosition {} => {
+            &Self::ClosePosition { size } => {
                 buf.push(4);
+                buf.extend_from_slice(&size.to_le_bytes());
             }
-            &Self::ExchangeWriterTokenForQuote {} => {
+            &Self::ExchangeWriterTokenForQuote { size } => {
                 buf.push(5);
+                buf.extend_from_slice(&size.to_le_bytes());
             }
         };
         buf
@@ -228,7 +247,6 @@ pub fn initialize_market(
     quote_asset_mint: &Pubkey,
     option_mint: &Pubkey,
     writer_token_mint: &Pubkey,
-    options_market: &Pubkey,
     underlying_asset_pool: &Pubkey,
     quote_asset_pool: &Pubkey,
     funding_account: &Pubkey,
@@ -236,8 +254,19 @@ pub fn initialize_market(
     quote_amount_per_contract: u64,
     expiration_unix_timestamp: UnixTimestamp,
 ) -> Result<Instruction, ProgramError> {
+    let (options_market_key, _no_duplication_bump) = Pubkey::find_program_address(
+        &[
+            &underlying_asset_mint.to_bytes(),
+            &quote_asset_mint.to_bytes(),
+            &underlying_amount_per_contract.to_le_bytes(),
+            &quote_amount_per_contract.to_le_bytes(),
+            &expiration_unix_timestamp.to_le_bytes(),
+        ],
+        &program_id,
+    );
     let (market_authority, bump_seed) =
-        Pubkey::find_program_address(&[&options_market.to_bytes()[..32]], &program_id);
+        Pubkey::find_program_address(&[&options_market_key.to_bytes()[..32]], &program_id);
+
     let data = OptionsInstruction::InitializeMarket {
         underlying_amount_per_contract,
         quote_amount_per_contract,
@@ -254,7 +283,6 @@ pub fn initialize_market(
         AccountMeta::new_readonly(*quote_asset_mint, false),
         AccountMeta::new(*option_mint, false),
         AccountMeta::new(*writer_token_mint, false),
-        AccountMeta::new(*options_market, false),
         AccountMeta::new_readonly(market_authority, false),
         AccountMeta::new(*underlying_asset_pool, false),
         AccountMeta::new(*quote_asset_pool, false),
@@ -262,6 +290,7 @@ pub fn initialize_market(
         AccountMeta::new_readonly(fee_owner_key::ID, false),
         AccountMeta::new(mint_fee_key, false),
         AccountMeta::new(exercise_fee_key, false),
+        AccountMeta::new(options_market_key, false),
         AccountMeta::new_readonly(sysvar::rent::id(), false),
         AccountMeta::new_readonly(spl_token::id(), false),
         AccountMeta::new_readonly(system_program::id(), false),
@@ -287,6 +316,7 @@ pub fn mint_covered_call(
     option_market: &Pubkey,
     underlying_asset_mint: &Pubkey,
     authority_pubkey: &Pubkey,
+    size: u64,
 ) -> Result<Instruction, ProgramError> {
     let fee_key = get_associated_token_address(&fee_owner_key::ID, underlying_asset_mint);
 
@@ -310,7 +340,7 @@ pub fn mint_covered_call(
     accounts.push(AccountMeta::new_readonly(sysvar::clock::id(), false));
     accounts.push(AccountMeta::new_readonly(system_program::id(), false));
 
-    let data = OptionsInstruction::MintCoveredCall {}.pack();
+    let data = OptionsInstruction::MintCoveredCall { size }.pack();
     Ok(Instruction {
         program_id: *program_id,
         data,
@@ -330,11 +360,12 @@ pub fn close_position(
     writer_token_source: &Pubkey,
     writer_token_source_authority: &Pubkey,
     underlying_asset_dest: &Pubkey,
+    size: u64,
 ) -> Result<Instruction, ProgramError> {
     let (market_authority, _bump_seed) =
         Pubkey::find_program_address(&[&options_market.to_bytes()[..32]], &program_id);
 
-    let data = OptionsInstruction::ClosePosition {}.pack();
+    let data = OptionsInstruction::ClosePosition { size }.pack();
 
     let mut accounts = Vec::with_capacity(11);
     accounts.push(AccountMeta::new_readonly(spl_token::id(), false));
@@ -371,10 +402,11 @@ pub fn close_post_expiration(
     writer_token_source: &Pubkey,
     writer_token_source_authority: &Pubkey,
     underlying_asset_dest: &Pubkey,
+    size: u64,
 ) -> Result<Instruction, ProgramError> {
     let (market_authority, _bump_seed) =
         Pubkey::find_program_address(&[&options_market.to_bytes()[..32]], &program_id);
-    let data = OptionsInstruction::ClosePostExpiration {}.pack();
+    let data = OptionsInstruction::ClosePostExpiration { size }.pack();
 
     let mut accounts = Vec::with_capacity(9);
     accounts.push(AccountMeta::new_readonly(*options_market, false));
@@ -411,10 +443,11 @@ pub fn exercise_covered_call(
     quote_asset_pool: &Pubkey,
     option_token_key: &Pubkey,
     option_token_authority: &Pubkey,
+    size: u64,
 ) -> Result<Instruction, ProgramError> {
     let (options_spl_authority_pubkey, _bump_seed) =
         Pubkey::find_program_address(&[&options_market.to_bytes()[..32]], &program_id);
-    let data = OptionsInstruction::ExerciseCoveredCall {}.pack();
+    let data = OptionsInstruction::ExerciseCoveredCall { size }.pack();
     let exercise_fee_key = get_associated_token_address(&fee_owner_key::ID, quote_asset_mint);
 
     let mut accounts = Vec::with_capacity(12);
@@ -455,10 +488,11 @@ pub fn exchange_writer_token_for_quote(
     writer_token_source_authority: &Pubkey,
     quote_asset_dest: &Pubkey,
     quote_asset_pool: &Pubkey,
+    size: u64,
 ) -> Result<Instruction, ProgramError> {
     let (option_market_authority, _bump_seed) =
         Pubkey::find_program_address(&[&options_market.to_bytes()[..32]], &program_id);
-    let data = OptionsInstruction::ExchangeWriterTokenForQuote {}.pack();
+    let data = OptionsInstruction::ExchangeWriterTokenForQuote { size }.pack();
 
     let mut accounts = Vec::with_capacity(8);
     accounts.push(AccountMeta::new_readonly(*options_market, false));
@@ -511,10 +545,11 @@ mod tests {
 
     #[test]
     fn test_pack_unpack_mint_covered_call() {
-        let check = OptionsInstruction::MintCoveredCall {};
+        let size = 2;
+        let check = OptionsInstruction::MintCoveredCall { size };
         let packed = check.pack();
         // add the tag to the expected buffer
-        let expect = Vec::from([1u8]);
+        let expect = Vec::from([1, 2, 0, 0, 0, 0, 0, 0, 0]);
         assert_eq!(packed, expect);
         let unpacked = OptionsInstruction::unpack(&expect).unwrap();
         assert_eq!(unpacked, check);
@@ -522,10 +557,11 @@ mod tests {
 
     #[test]
     fn test_pack_unpack_exercise_covered_call() {
-        let check = OptionsInstruction::ExerciseCoveredCall {};
+        let size = 2;
+        let check = OptionsInstruction::ExerciseCoveredCall { size };
         let packed = check.pack();
         // add the tag to the expected buffer
-        let expect = Vec::from([2u8]);
+        let expect = Vec::from([2, 2, 0, 0, 0, 0, 0, 0, 0]);
         assert_eq!(packed, expect);
         let unpacked = OptionsInstruction::unpack(&expect).unwrap();
         assert_eq!(unpacked, check);
@@ -533,10 +569,11 @@ mod tests {
 
     #[test]
     fn test_pack_unpack_close_post_expiration() {
-        let check = OptionsInstruction::ClosePostExpiration {};
+        let size = 2;
+        let check = OptionsInstruction::ClosePostExpiration { size };
         let packed = check.pack();
         // add the tag to the expected buffer
-        let expect = Vec::from([3u8]);
+        let expect = Vec::from([3, 2, 0, 0, 0, 0, 0, 0, 0]);
         assert_eq!(packed, expect);
         let unpacked = OptionsInstruction::unpack(&expect).unwrap();
         assert_eq!(unpacked, check);
@@ -544,10 +581,11 @@ mod tests {
 
     #[test]
     fn test_pack_unpack_close_position() {
-        let check = OptionsInstruction::ClosePosition {};
+        let size = 2;
+        let check = OptionsInstruction::ClosePosition { size };
         let packed = check.pack();
         // add the tag to the expected buffer
-        let expect = Vec::from([4u8]);
+        let expect = Vec::from([4, 2, 0, 0, 0, 0, 0, 0, 0]);
         assert_eq!(packed, expect);
         let unpacked = OptionsInstruction::unpack(&expect).unwrap();
         assert_eq!(unpacked, check);
@@ -555,10 +593,11 @@ mod tests {
 
     #[test]
     fn test_pack_unpack_exchange_writer_token_for_quote() {
-        let check = OptionsInstruction::ExchangeWriterTokenForQuote {};
+        let size = 2;
+        let check = OptionsInstruction::ExchangeWriterTokenForQuote { size };
         let packed = check.pack();
         // add the tag to the expected buffer
-        let expect = Vec::from([5u8]);
+        let expect = Vec::from([5, 2, 0, 0, 0, 0, 0, 0, 0]);
         assert_eq!(packed, expect);
         let unpacked = OptionsInstruction::unpack(&expect).unwrap();
         assert_eq!(unpacked, check);
